package de.unikoblenz.fgbas.bs.bpm.iot;

import de.unikoblenz.fgbas.bs.bpm.iot.service.RfidProductService;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.util.EntityUtils;
import org.camunda.bpm.engine.delegate.DelegateExecution;
import org.camunda.bpm.engine.delegate.JavaDelegate;
import org.camunda.bpm.engine.impl.util.json.JSONArray;
import org.camunda.bpm.engine.impl.util.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.apache.http.impl.client.HttpClients;

public class RfidTimerReaderService implements JavaDelegate {

  public static final String GET_HTTP_RFID_API = "http://bs19-team34.bas.uni-koblenz.de/api/rfid.php?action=get";

  private final Logger LOGGER = LoggerFactory.getLogger(RfidTimerReaderService.class);

  public void execute(DelegateExecution delegateExecution) throws Exception {
    LOGGER.info("Amazing IoT api call....");
    Optional<HttpResponse> response = sendHTTPRequest();
    if (response.isPresent()) {
      String result = EntityUtils.toString(response.get().getEntity());
      LOGGER.info("Response: " + result);
      JSONArray listOfRfidTags = new JSONArray(result);
      for (int i = 0; i < listOfRfidTags.length(); i++) {
        JSONObject jo = listOfRfidTags.getJSONObject(i);
        String rfidTag = jo.getString("tagId");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date = sdf.parse(jo.getString("timeStamp"));
        LOGGER.info("Starting product review with rfid tag {" + rfidTag + "} Date:" + date.toString());
        // Create new Process
        Map<String, Object> variables = new HashMap<String, Object>();
        variables.put("rfidTag", rfidTag);
        variables.put("product", RfidProductService.getProductByRfidTag(rfidTag));
        delegateExecution.getProcessEngineServices().getRuntimeService()
            .startProcessInstanceByKey("Product_Review_Process", variables);

      }
    }
  }

  private Optional<HttpResponse> sendHTTPRequest() {
    LOGGER.info("Message sending..");
    HttpClient client = HttpClients.createDefault();
    HttpGet get = new HttpGet(GET_HTTP_RFID_API);
    get.setHeader("content-type", "application/json");
    try {
      return Optional.of(client.execute(get));
    } catch (Exception e) {
      return Optional.empty();
    }
  }
}
